from flask import request, jsonify, abort
import json
import sqlite3

from . import sqlite as db
from . import utilities as ut

from . import auth


@auth.request_is_authenticated
def tables(context):

    conn = db.get_db(context['sqlitepath'])
    
    if request.method == 'GET':
        tables = db.get_tables(conn)
        return jsonify(tables)

    if request.method == 'POST':
        data = json.loads(request.data)
        name = data['name']
        schema = data['schema']
        string_schema = ', '.join(list(map( lambda x: f'{x[0]} {x[1]}', schema.items())))
        query = f'CREATE TABLE {name} ({string_schema});'
        c = conn.cursor()
        c.execute(query)
        conn.commit()
        return '', 200


@auth.request_is_authenticated
def table(context, name):
    
    conn = db.get_db(context['sqlitepath'])

    if name in db.get_tables(conn):

        if request.method == 'GET':

            conn.row_factory = sqlite3.Row
            c = conn.cursor()

            columns, args = ut.parse_args(request.args)

            if not columns:
                columns = '*'
            
            if args:
                conditions = ' AND '.join(map(lambda x: f'{x}=:{x}', args.keys()))
                query = f'SELECT {columns} FROM {name} WHERE {conditions};'
                c.execute(query, args)

            else:
                query = f'SELECT {columns} FROM {name};'
                c.execute(query)
            
            result = []
            for row in c.fetchall():
                result.append(dict(row))
            
            return jsonify(result)

        if request.method == 'POST':
            data = json.loads(request.data)
            query = f'''INSERT INTO {name} ({", ".join([k for k in data.keys()])}) VALUES ({", ".join(["?" for _ in data])})'''
            c = conn.cursor()
            c.execute(query, list(data.values()))
            conn.commit()
            return '', 200

        if request.method == 'DELETE':
            query = f'DROP TABLE {name};'
            c = conn.cursor()
            c.execute(query)
            return ''

    else:
        abort(404)




from flask import g, jsonify

from . import views as v


class CloudSQL():

    def __init__(self, dbpath, url_prefix='', api_key=None):
        self.url_prefix = url_prefix
        self.db_context = {'sqlitepath': dbpath}
        if api_key:
            self.db_context['api_key'] =  api_key


    def serve(self, app):
        
        @app.route(f'{self.url_prefix}/tables', methods=['GET', 'POST'])
        def tables():
            return v.tables(self.db_context)

        @app.route(f'{self.url_prefix}/table/<name>', methods=['GET', 'POST', 'DELETE'])
        def table(name):
            return v.table(self.db_context, name)

        @app.teardown_appcontext
        def close_connection(exception):
            db = getattr(g, '_database', None)
            if db is not None:
                db.close()

        @app.errorhandler(Exception)
        def handle_error(e):
            print(e)
            return jsonify(error=str(e))



import sqlite3
from flask import g

# sqlite interfaces
def get_db(database):
    db = getattr(g, '_database', None)
    if db is None:
        db = g._database = sqlite3.connect(database)
    return db

def get_tables(conn):
    tables = []
    for row in conn.execute("SELECT name FROM sqlite_master WHERE type='table';").fetchall():
        tables.append(row[0])
    return tables


from flask import request, abort

def request_is_authenticated(func):
    def wrapper(*args, **kwargs):

        key = args[0].get('api_key')
        request_key = request.args.get('api_key')

        if key:
            if request_key != key:
                abort(403)

        return func(*args, **kwargs)

    return wrapper